@@ -1,5 +1,6 @@
 import inspect
 from importlib import import_module
+from inspect import cleandoc
 from pathlib import Path
 
 from django.apps import apps
@@ -14,11 +15,11 @@ from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
 from django.db import models
 from django.http import Http404
 from django.template.engine import Engine
-from django.urls import get_mod_func, get_resolver, get_urlconf, reverse
+from django.urls import get_mod_func, get_resolver, get_urlconf
 from django.utils.decorators import method_decorator
 from django.utils.inspect import (
-    func_accepts_kwargs, func_accepts_var_args, func_has_no_args,
-    get_func_full_args,
+    func_accepts_kwargs, func_accepts_var_args, get_func_full_args,
+    method_has_no_args,
 )
 from django.utils.translation import gettext as _
 from django.views.generic import TemplateView
@@ -44,7 +45,6 @@ class BaseAdminDocsView(TemplateView):
     def get_context_data(self, **kwargs):
         return super().get_context_data(**{
             **kwargs,
-            'root_path': reverse('admin:index'),
             **admin.site.each_context(self.request),
         })
 
@@ -52,14 +52,6 @@ class BaseAdminDocsView(TemplateView):
 class BookmarkletsView(BaseAdminDocsView):
     template_name = 'admin_doc/bookmarklets.html'
 
-    def get_context_data(self, **kwargs):
-        context = super().get_context_data(**kwargs)
-        context.update({
-            'admin_url': "%s://%s%s" % (
-                self.request.scheme, self.request.get_host(), context['root_path'])
-        })
-        return context
-
 
 class TemplateTagIndexView(BaseAdminDocsView):
     template_name = 'admin_doc/template_tag_index.html'
@@ -135,7 +127,7 @@ class ViewIndexView(BaseAdminDocsView):
                 'full_name': get_view_name(func),
                 'url': simplify_regex(regex),
                 'url_name': ':'.join((namespace or []) + (name and [name] or [])),
-                'namespace': ':'.join((namespace or [])),
+                'namespace': ':'.join(namespace or []),
                 'name': name,
             })
         return super().get_context_data(**{**kwargs, 'views': views})
@@ -256,7 +248,7 @@ class ModelDetailView(BaseAdminDocsView):
         methods = []
         # Gather model methods.
         for func_name, func in model.__dict__.items():
-            if inspect.isfunction(func):
+            if inspect.isfunction(func) or isinstance(func, property):
                 try:
                     for exclude in MODEL_METHODS_EXCLUDE:
                         if func_name.startswith(exclude):
@@ -265,11 +257,17 @@ class ModelDetailView(BaseAdminDocsView):
                     continue
                 verbose = func.__doc__
                 verbose = verbose and (
-                    utils.parse_rst(utils.trim_docstring(verbose), 'model', _('model:') + opts.model_name)
+                    utils.parse_rst(cleandoc(verbose), 'model', _('model:') + opts.model_name)
                 )
-                # If a method has no arguments, show it as a 'field', otherwise
-                # as a 'method with arguments'.
-                if func_has_no_args(func) and not func_accepts_kwargs(func) and not func_accepts_var_args(func):
+                # Show properties and methods without arguments as fields.
+                # Otherwise, show as a 'method with arguments'.
+                if isinstance(func, property):
+                    fields.append({
+                        'name': func_name,
+                        'data_type': get_return_data_type(func_name),
+                        'verbose': verbose or ''
+                    })
+                elif method_has_no_args(func) and not func_accepts_kwargs(func) and not func_accepts_var_args(func):
                     fields.append({
                         'name': func_name,
                         'data_type': get_return_data_type(func_name),
@@ -281,7 +279,7 @@ class ModelDetailView(BaseAdminDocsView):
                     # join it with '='. Use repr() so that strings will be
                     # correctly displayed.
                     print_arguments = ', '.join([
-                        '='.join(list(arg_el[:1]) + [repr(el) for el in arg_el[1:]])
+                        '='.join([arg_el[0], *map(repr, arg_el[1:])])
                         for arg_el in arguments
                     ])
                     methods.append({
@@ -309,7 +307,7 @@ class ModelDetailView(BaseAdminDocsView):
             })
         return super().get_context_data(**{
             **kwargs,
-            'name': '%s.%s' % (opts.app_label, opts.object_name),
+            'name': opts.label,
             'summary': title,
             'description': body,
             'fields': fields,
@@ -333,8 +331,7 @@ class TemplateDetailView(BaseAdminDocsView):
             for index, directory in enumerate(default_engine.dirs):
                 template_file = Path(directory) / template
                 if template_file.exists():
-                    with template_file.open() as f:
-                        template_contents = f.read()
+                    template_contents = template_file.read_text()
                 else:
                     template_contents = ''
                 templates.append({
