@@ -1,19 +1,12 @@
-# Since this package contains a "django" module, this is required on Python 2.
-from __future__ import absolute_import
-
-import sys
-import warnings
 from importlib import import_module
 from pkgutil import walk_packages
 
 from django.apps import apps
 from django.conf import settings
 from django.template import TemplateDoesNotExist
-from django.template.context import Context, RequestContext, make_context
-from django.template.engine import Engine, _dirs_undefined
+from django.template.context import make_context
+from django.template.engine import Engine
 from django.template.library import InvalidTemplateLibrary
-from django.utils import six
-from django.utils.deprecation import RemovedInDjango110Warning
 
 from .base import BaseEngine
 
@@ -25,19 +18,20 @@ class DjangoTemplates(BaseEngine):
     def __init__(self, params):
         params = params.copy()
         options = params.pop('OPTIONS').copy()
+        options.setdefault('autoescape', True)
         options.setdefault('debug', settings.DEBUG)
-        options.setdefault('file_charset', settings.FILE_CHARSET)
+        options.setdefault('file_charset', 'utf-8')
         libraries = options.get('libraries', {})
         options['libraries'] = self.get_templatetag_libraries(libraries)
-        super(DjangoTemplates, self).__init__(params)
+        super().__init__(params)
         self.engine = Engine(self.dirs, self.app_dirs, **options)
 
     def from_string(self, template_code):
         return Template(self.engine.from_string(template_code), self)
 
-    def get_template(self, template_name, dirs=_dirs_undefined):
+    def get_template(self, template_name):
         try:
-            return Template(self.engine.get_template(template_name, dirs), self)
+            return Template(self.engine.get_template(template_name), self)
         except TemplateDoesNotExist as exc:
             reraise(exc, self)
 
@@ -51,7 +45,7 @@ class DjangoTemplates(BaseEngine):
         return libraries
 
 
-class Template(object):
+class Template:
 
     def __init__(self, template, backend):
         self.template = template
@@ -62,35 +56,7 @@ class Template(object):
         return self.template.origin
 
     def render(self, context=None, request=None):
-        # A deprecation path is required here to cover the following usage:
-        # >>> from django.template import Context
-        # >>> from django.template.loader import get_template
-        # >>> template = get_template('hello.html')
-        # >>> template.render(Context({'name': 'world'}))
-        # In Django 1.7 get_template() returned a django.template.Template.
-        # In Django 1.8 it returns a django.template.backends.django.Template.
-        # In Django 1.10 the isinstance checks should be removed. If passing a
-        # Context or a RequestContext works by accident, it won't be an issue
-        # per se, but it won't be officially supported either.
-        if isinstance(context, RequestContext):
-            if request is not None and request is not context.request:
-                raise ValueError(
-                    "render() was called with a RequestContext and a request "
-                    "argument which refer to different requests. Make sure "
-                    "that the context argument is a dict or at least that "
-                    "the two arguments refer to the same request.")
-            warnings.warn(
-                "render() must be called with a dict, not a RequestContext.",
-                RemovedInDjango110Warning, stacklevel=2)
-
-        elif isinstance(context, Context):
-            warnings.warn(
-                "render() must be called with a dict, not a Context.",
-                RemovedInDjango110Warning, stacklevel=2)
-
-        else:
-            context = make_context(context, request)
-
+        context = make_context(context, request, autoescape=self.backend.engine.autoescape)
         try:
             return self.template.render(context)
         except TemplateDoesNotExist as exc:
@@ -115,7 +81,7 @@ def reraise(exc, backend):
     Reraise TemplateDoesNotExist while maintaining template debug information.
     """
     new = copy_exception(exc, backend)
-    six.reraise(exc.__class__, new, sys.exc_info()[2])
+    raise new from exc
 
 
 def get_installed_libraries():
